@@ -17,6 +17,10 @@ mkdir -p $ODL_DIR
 # Import utility functions
 source $TOP_DIR/functions
 source $NETWORKING_ODL_DIR/devstack/functions
+source $TOP_DIR/lib/neutron
+
+# TODO(manjeets) Remove this once lib/neutron is
+# adapted for grenade job.
 source $TOP_DIR/lib/neutron-legacy
 
 # Import bridge data
@@ -67,8 +71,8 @@ if is_service_enabled odl-server; then
     fi
 
     if [[ "$1" == "clean" ]]; then
-        # no-op
-        :
+        stop_opendaylight
+        cleanup_opendaylight
     fi
 fi
 
@@ -78,11 +82,12 @@ if is_service_enabled odl-compute; then
         install_opendaylight_compute
     elif [[ "$1" == "stack" && "$2" == "post-config" ]]; then
         if is_service_enabled nova; then
-            create_nova_conf_neutron
+            configure_neutron_nova
         fi
         bind_opendaylight_controller
-        sudo ovs-vsctl --may-exist add-br $OVS_BR
-        wait_for_active_bridge $OVS_BR $ODL_RETRY_SLEEP_INTERVAL $ODL_BOOT_WAIT
+        if [[ -z "$ODL_DONT_WAIT_OVS_BR" ]]; then
+            wait_for_active_bridge $OVS_BR $ODL_RETRY_SLEEP_INTERVAL $ODL_BOOT_WAIT
+        fi
 
         # L3 needs to be configured only for netvirt-ovsdb - in netvirt-vpnservice L3 is configured
         # by provider_mappings, and the provider mappings are added to br-int by default
