@@ -15,35 +15,44 @@
 import datetime
 
 from sqlalchemy import asc
+from sqlalchemy import bindparam
+from sqlalchemy.ext import baked
 from sqlalchemy import func
 from sqlalchemy import or_
 from sqlalchemy.orm import aliased
 
-from networking_odl.common import constants as odl_const
-from networking_odl.db import models
+from oslo_log import log as logging
 
 from neutron.db import api as db_api
 
-from oslo_db import api as oslo_db_api
-from oslo_log import log as logging
-
+from networking_odl.common import constants as odl_const
+from networking_odl.db import models
 
 LOG = logging.getLogger(__name__)
 
+bakery = baked.bakery()
+
 
 def get_pending_or_processing_ops(session, object_uuid, operation=None):
-    q = session.query(models.OpenDaylightJournal).filter(
+    # NOTE (sai): For performance reasons, we expect this method to use baked
+    # query (http://docs.sqlalchemy.org/en/latest/orm/extensions/baked.html)
+    baked_query = bakery(lambda s: s.query(
+        models.OpenDaylightJournal))
+    baked_query += lambda q: q.filter(
         or_(models.OpenDaylightJournal.state == odl_const.PENDING,
             models.OpenDaylightJournal.state == odl_const.PROCESSING),
-        models.OpenDaylightJournal.object_uuid == object_uuid)
-
+        models.OpenDaylightJournal.object_uuid == bindparam('uuid'))
     if operation:
         if isinstance(operation, (list, tuple)):
-            q = q.filter(models.OpenDaylightJournal.operation.in_(operation))
+            baked_query += lambda q: q.filter(
+                models.OpenDaylightJournal.operation.in_(bindparam('op',
+                                                         expanding=True)))
         else:
-            q = q.filter(models.OpenDaylightJournal.operation == operation)
+            baked_query += lambda q: q.filter(
+                models.OpenDaylightJournal.operation == bindparam('op'))
 
-    return q.all()
+    return baked_query(session).params(
+        uuid=object_uuid, op=operation).all()
 
 
 def get_pending_delete_ops_with_parent(session, object_type, parent_id):
@@ -72,19 +81,25 @@ def get_all_db_rows_by_state(session, state):
 # of them will get a deadlock from Galera and will have to retry the operation.
 @db_api.retry_db_errors
 def get_oldest_pending_db_row_with_lock(session):
-    with session.begin():
+    # NOTE (sai): For performance reasons, we expect this method to use baked
+    # query (http://docs.sqlalchemy.org/en/latest/orm/extensions/baked.html)
+    with db_api.autonested_transaction(session):
         journal_dep = aliased(models.OpenDaylightJournal)
-        dep_query = session.query(journal_dep).filter(
-            models.OpenDaylightJournal.seqnum == journal_dep.seqnum
-        ).outerjoin(
-            journal_dep.depending_on, aliased=True).filter(
+        dep_query = bakery(lambda s1: s1.query(journal_dep))
+        dep_query += lambda q: q.filter(
+            models.OpenDaylightJournal.seqnum == journal_dep.seqnum)
+        dep_query += lambda q: q.outerjoin(
+            journal_dep.depending_on, aliased=True)
+        dep_query += lambda q: q.filter(
             or_(models.OpenDaylightJournal.state == odl_const.PENDING,
                 models.OpenDaylightJournal.state == odl_const.PROCESSING))
-        row = session.query(models.OpenDaylightJournal).filter(
+        row = bakery(lambda s2: s2.query(models.OpenDaylightJournal))
+        row += lambda q: q.filter(
             models.OpenDaylightJournal.state == odl_const.PENDING,
-            ~ dep_query.exists()
-        ).order_by(
-            asc(models.OpenDaylightJournal.last_retried)).first()
+            ~ (dep_query._as_query(q.session)).exists())
+        row += lambda q: q.order_by(
+            asc(models.OpenDaylightJournal.last_retried))
+        row = row(session).first()
         if row:
             update_db_row_state(session, row, odl_const.PROCESSING)
 
@@ -100,11 +115,11 @@ def delete_dependency(session, entry):
     session.expire_all()
 
 
-@oslo_db_api.wrap_db_retry(max_retries=db_api.MAX_RETRIES)
-def update_db_row_state(session, row, state):
+def update_db_row_state(session, row, state, flush=True):
     row.state = state
     session.merge(row)
-    session.flush()
+    if flush:
+        session.flush()
 
 
 def update_pending_db_row_retry(session, row, retry_count):
@@ -115,19 +130,16 @@ def update_pending_db_row_retry(session, row, retry_count):
         update_db_row_state(session, row, odl_const.PENDING)
 
 
-# This function is currently not used.
-# Deleted resources are marked as 'deleted' in the database.
-@oslo_db_api.wrap_db_retry(max_retries=db_api.MAX_RETRIES)
-def delete_row(session, row=None, row_id=None):
+def delete_row(session, row=None, row_id=None, flush=True):
     if row_id:
         row = session.query(models.OpenDaylightJournal).filter_by(
-            id=row_id).one()
+            seqnum=row_id).one()
     if row:
         session.delete(row)
-        session.flush()
+        if flush:
+            session.flush()
 
 
-@oslo_db_api.wrap_db_retry(max_retries=db_api.MAX_RETRIES)
 def create_pending_row(session, object_type, object_uuid,
                        operation, data, depending_on=None):
     if depending_on is None:
@@ -135,18 +147,17 @@ def create_pending_row(session, object_type, object_uuid,
     row = models.OpenDaylightJournal(object_type=object_type,
                                      object_uuid=object_uuid,
                                      operation=operation, data=data,
-                                     created_at=func.now(),
                                      state=odl_const.PENDING,
                                      depending_on=depending_on)
     session.add(row)
     # Keep session flush for unit tests. NOOP for L2/L3 events since calls are
     # made inside database session transaction with subtransactions=True.
     session.flush()
+    return row
 
 
-@db_api.retry_db_errors
 def delete_pending_rows(session, operations_to_delete):
-    with session.begin():
+    with db_api.autonested_transaction(session):
         session.query(models.OpenDaylightJournal).filter(
             models.OpenDaylightJournal.operation.in_(operations_to_delete),
             models.OpenDaylightJournal.state == odl_const.PENDING).delete(
@@ -154,18 +165,16 @@ def delete_pending_rows(session, operations_to_delete):
         session.expire_all()
 
 
-@db_api.retry_db_errors
 def _update_periodic_task_state(session, expected_state, state, task):
-    with session.begin():
-        row = session.query(models.OpenDaylightPeriodicTask).filter_by(
-            state=expected_state,
-            task=task).with_for_update().one_or_none()
+    row = session.query(models.OpenDaylightPeriodicTask).filter_by(
+        state=expected_state,
+        task=task).with_for_update().one_or_none()
 
-        if row is None:
-            return False
+    if row is None:
+        return False
 
-        row.state = state
-        return True
+    row.state = state
+    return True
 
 
 def was_periodic_task_executed_recently(session, task, interval):
@@ -199,29 +208,36 @@ def update_periodic_task(session, task, operation=None):
     if operation:
         op_text = operation.__name__
 
-    with session.begin():
-        row = session.query(models.OpenDaylightPeriodicTask).filter_by(
-            task=task).one()
-        row.processing_operation = op_text
+    row = session.query(models.OpenDaylightPeriodicTask).filter_by(
+        task=task).one()
+    row.processing_operation = op_text
 
 
 def delete_rows_by_state_and_time(session, state, time_delta):
-    with session.begin():
+    # NOTE(mpeterson): The reason behind deleting one-by-one is that InnoDB
+    # ignores the WHERE clause to issue a LOCK when executing a DELETE. By
+    # executing each operation indepently, we minimize exposures to DEADLOCKS.
+    with db_api.autonested_transaction(session):
         now = session.execute(func.now()).scalar()
-        session.query(models.OpenDaylightJournal).filter(
+        rows = session.query(models.OpenDaylightJournal).filter(
             models.OpenDaylightJournal.state == state,
-            models.OpenDaylightJournal.last_retried < now - time_delta).delete(
-            synchronize_session=False)
+            models.OpenDaylightJournal.last_retried < now - time_delta).all()
+        for row in rows:
+            delete_row(session, row, flush=False)
         session.expire_all()
 
 
 def reset_processing_rows(session, max_timedelta):
-    with session.begin():
+    # NOTE(mpeterson): The reason behind updating one-by-one is that InnoDB
+    # ignores the WHERE clause to issue a LOCK when executing an UPDATE. By
+    # executing each operation indepently, we minimize exposures to DEADLOCKS.
+    with db_api.autonested_transaction(session):
         now = session.execute(func.now()).scalar()
         max_timedelta = datetime.timedelta(seconds=max_timedelta)
         rows = session.query(models.OpenDaylightJournal).filter(
             models.OpenDaylightJournal.last_retried < now - max_timedelta,
-            models.OpenDaylightJournal.state == odl_const.PROCESSING,
-        ).update({'state': odl_const.PENDING})
+            models.OpenDaylightJournal.state == odl_const.PROCESSING).all()
+        for row in rows:
+            update_db_row_state(session, row, odl_const.PENDING, flush=False)
 
-    return rows
+    return len(rows)
