@@ -497,21 +497,29 @@ class TestPseudoAgentDBBindingController(base.DietTestCase):
             self.test_valid_segment[api.ID], pass_vif_type,
             pass_vif_details, status=n_const.PORT_STATUS_ACTIVE)
 
-    def _test_bind_port_failed_when_agent_dead(self, hconfig):
-        hconfig['alive'] = False
+    def _test_bind_port_succeed_when_agent_status(self, hconfig, agent_status):
+        hconfig['alive'] = agent_status
         port_context = self._fake_port_context(
             fake_segments=[self.test_invalid_segment, self.test_valid_segment],
             host_agents=[hconfig])
         self.mgr.bind_port(port_context)
-        port_context.set_binding.assert_not_called()
+        port_context.set_binding.assert_called()
 
-    def test_bind_port_failed_when_agent_dead_vpp(self):
+    def test_bind_port_succeed_when_agent_dead_vpp(self):
         hconfig = deepcopy(self.sample_hconf_str_tmpl_subs_vpp)
-        self._test_bind_port_failed_when_agent_dead(hconfig)
+        self._test_bind_port_succeed_when_agent_status(hconfig, False)
 
-    def test_bind_port_failed_when_agent_dead_ovs(self):
+    def test_bind_port_succeed_when_agent_dead_ovs(self):
         hconfig = deepcopy(self.sample_hconf_str_tmpl_subs_ovs)
-        self._test_bind_port_failed_when_agent_dead(hconfig)
+        self._test_bind_port_succeed_when_agent_status(hconfig, False)
+
+    def test_bind_port_succeed_when_agent_alive_vpp(self):
+        hconfig = deepcopy(self.sample_hconf_str_tmpl_subs_vpp)
+        self._test_bind_port_succeed_when_agent_status(hconfig, True)
+
+    def test_bind_port_succeed_when_agent_alive_ovs(self):
+        hconfig = deepcopy(self.sample_hconf_str_tmpl_subs_ovs)
+        self._test_bind_port_succeed_when_agent_status(hconfig, True)
 
     def test_bind_port_with_vif_type_vhost_user_vpp(self):
         """test bind_port with vpp."""
